/**
 * Copyright (c) 2000-2013 Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.cache.ehcache;

import com.liferay.portal.kernel.bean.PortalBeanLocatorUtil;
import com.liferay.portal.kernel.cluster.Address;
import com.liferay.portal.kernel.cluster.ClusterExecutorUtil;
import com.liferay.portal.kernel.cluster.ClusterLinkUtil;
import com.liferay.portal.kernel.cluster.ClusterNodeResponse;
import com.liferay.portal.kernel.cluster.ClusterRequest;
import com.liferay.portal.kernel.cluster.FutureClusterResponses;
import com.liferay.portal.kernel.concurrent.ConcurrentHashSet;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.io.AnnotatedObjectInputStream;
import com.liferay.portal.kernel.io.ProtectedObjectInputStream;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.messaging.DestinationNames;
import com.liferay.portal.kernel.messaging.HotDeployMessageListener;
import com.liferay.portal.kernel.messaging.Message;
import com.liferay.portal.kernel.messaging.MessageBusUtil;
import com.liferay.portal.kernel.servlet.ServletContextPool;
import com.liferay.portal.kernel.util.ClassLoaderPool;
import com.liferay.portal.kernel.util.MethodHandler;
import com.liferay.portal.kernel.util.MethodKey;
import com.liferay.portal.kernel.util.SocketUtil;
import com.liferay.portal.kernel.util.SocketUtil.ServerSocketConfigurator;
import com.liferay.portal.util.PropsValues;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;

import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketAddress;
import java.net.SocketException;
import java.net.SocketTimeoutException;

import java.nio.channels.ServerSocketChannel;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.TimeUnit;

import net.sf.ehcache.CacheManager;
import net.sf.ehcache.Ehcache;
import net.sf.ehcache.Element;

/**
 * @author Shuyang Zhou
 * @author Sherry Yang
 */
public class EhcacheStreamBootstrapHelpUtil {

	public static SocketAddress createServerSocketFromCluster(
			List<String> cacheNames)
		throws Exception {

		ServerSocketChannel serverSocketChannel =
			SocketUtil.createServerSocketChannel(
				ClusterLinkUtil.getBindInetAddress(),
				PropsValues.EHCACHE_SOCKET_START_PORT,
				_serverSocketConfigurator);

		ServerSocket serverSocket = serverSocketChannel.socket();

		EhcachePortalCacheManager<?, ?> ehcachePortalCacheManager =
			(EhcachePortalCacheManager<?, ?>)PortalBeanLocatorUtil.locate(
				_BEAN_NAME_MULTI_VM_PORTAL_CACHE_MANAGER);

		CacheManager cacheManager =
			ehcachePortalCacheManager.getEhcacheManager();

		EhcacheStreamServerThread ehcacheStreamServerThread =
			new EhcacheStreamServerThread(
				serverSocket, cacheManager, cacheNames);

		ehcacheStreamServerThread.start();

		return serverSocket.getLocalSocketAddress();
	}

	protected static void loadCachesFromCluster(Ehcache... ehcaches)
		throws Exception {

		List<Address> clusterNodeAddresses =
			ClusterExecutorUtil.getClusterNodeAddresses();

		if (_log.isInfoEnabled()) {
			_log.info("Cluster node addresses " + clusterNodeAddresses);
		}

		if (clusterNodeAddresses.size() <= 1) {
			if (_log.isDebugEnabled()) {
				_log.debug(
					"Not loading cache from cluster because a cluster peer " +
						"was not found");
			}

			return;
		}

		EhcachePortalCacheManager<?, ?> ehcachePortalCacheManager =
			(EhcachePortalCacheManager<?, ?>)PortalBeanLocatorUtil.locate(
				_BEAN_NAME_MULTI_VM_PORTAL_CACHE_MANAGER);

		CacheManager cacheManager =
			ehcachePortalCacheManager.getEhcacheManager();

		List<String> cacheNames = new ArrayList<String>();

		for (Ehcache ehcache : ehcaches) {
			if (cacheManager == ehcache.getCacheManager()) {
				cacheNames.add(ehcache.getName());
			}
		}

		ClusterRequest clusterRequest = ClusterRequest.createMulticastRequest(
			new MethodHandler(
				_createServerSocketFromClusterMethodKey, cacheNames),
			true);

		FutureClusterResponses futureClusterResponses =
			ClusterExecutorUtil.execute(clusterRequest);

		BlockingQueue<ClusterNodeResponse> clusterNodeResponses =
			futureClusterResponses.getPartialResults();

		ClusterNodeResponse clusterNodeResponse = null;

		try {
			clusterNodeResponse = clusterNodeResponses.poll(
				PropsValues.CLUSTER_LINK_NODE_BOOTUP_RESPONSE_TIMEOUT,
				TimeUnit.MILLISECONDS);
		}
		catch (InterruptedException ie) {
			return;
		}

		if (clusterNodeResponse == null) {
			if (_log.isWarnEnabled()) {
				_log.warn(
					"Unable to load cache from the cluster because there " +
						"was no peer response");
			}

			return;
		}

		Socket socket = null;

		try {
			SocketAddress remoteSocketAddress =
				(SocketAddress)clusterNodeResponse.getResult();

			if (remoteSocketAddress == null) {
				_log.error(
					"Cluster peer " + clusterNodeResponse.getClusterNode() +
						" responded with a null socket address");

				return;
			}

			socket = new Socket();

			socket.connect(remoteSocketAddress);

			socket.shutdownOutput();

			ObjectInputStream objectInputStream =
				new ProtectedObjectInputStream(socket.getInputStream());

			Ehcache ehcache = null;

			try {
				while (true) {
					Object object = objectInputStream.readObject();

					if (object instanceof EhcacheElement) {
						EhcacheElement ehcacheElement = (EhcacheElement)object;

						Element element = ehcacheElement.toElement();

						if (element != null) {
							ehcache.put(element, true);
						}
						else {
							Set<EhcacheElement> ehcacheElements =
								_unloadedEhcaches.get(ehcache.getName());

							if (ehcacheElements == null) {
								ehcacheElements = _unloadedEhcaches.putIfAbsent(
									ehcache.getName(),
									new ConcurrentHashSet<EhcacheElement>());

								if (ehcacheElements == null) {
									ehcacheElements = _unloadedEhcaches.get(
										ehcache.getName());
								}
							}

							ehcacheElements.add(ehcacheElement);
						}
					}
					else if (object instanceof String) {
						if (_COMMAND_SOCKET_CLOSE.equals(object)) {
							break;
						}

						EhcacheStreamBootstrapCacheLoader.setSkip();

						try {
							ehcache = cacheManager.addCacheIfAbsent(
								(String)object);
						}
						finally {
							EhcacheStreamBootstrapCacheLoader.resetSkip();
						}
					}
					else {
						throw new SystemException(
							"Socket input stream returned invalid object " +
								object);
					}
				}
			}
			finally {
				if (objectInputStream != null) {
					objectInputStream.close();
				}
			}
		}
		finally {
			if (socket != null) {
				socket.close();
			}
		}
	}

	protected static void registerHotDeployMessaageListener() {
		EhcachePortalCacheManager<?, ?>
			ehcachePortalCacheManager =
				(EhcachePortalCacheManager<?, ?>)
					PortalBeanLocatorUtil.locate(
						_BEAN_NAME_MULTI_VM_PORTAL_CACHE_MANAGER);

		MessageBusUtil.registerMessageListener(
			DestinationNames.HOT_DEPLOY,
			new EhcacheHotDeployMessageListener(
				ehcachePortalCacheManager.getEhcacheManager()));
	}

	private static final String _BEAN_NAME_MULTI_VM_PORTAL_CACHE_MANAGER =
		"com.liferay.portal.kernel.cache.MultiVMPortalCacheManager";

	private static final String _COMMAND_SOCKET_CLOSE = "${SOCKET_CLOSE}";

	private static Log _log = LogFactoryUtil.getLog(
		EhcacheStreamBootstrapHelpUtil.class);

	private static MethodKey _createServerSocketFromClusterMethodKey =
		new MethodKey(
			EhcacheStreamBootstrapHelpUtil.class,
			"createServerSocketFromCluster", List.class);
	private static ServerSocketConfigurator _serverSocketConfigurator =
		new SocketCacheServerSocketConfiguration();

	private static ConcurrentMap<String, Set<EhcacheElement>>
		_unloadedEhcaches =
			new ConcurrentHashMap<String, Set<EhcacheElement>>();

	private static class EhcacheElement implements Externalizable {

		public EhcacheElement() {
		}

		public EhcacheElement(Serializable key, Serializable value) {
			_key = key;
			_value = value;
		}

		@Override
		public void readExternal(ObjectInput objectInput) throws IOException {
			try {
				_dependentContextNames = (Set<String>)objectInput.readObject();

				_data = new byte[objectInput.readInt()];

				objectInput.readFully(_data);
			}
			catch (ClassNotFoundException cnfe) {
				_log.error("Unable to deserialize object", cnfe);
			}
		}

		public Element toElement() throws IOException {
			if (_data != null) {
				for (String dependentContextName :_dependentContextNames) {
					if (!ServletContextPool.containsKey(dependentContextName)) {
						return null;
					}
				}

				try {
					ObjectInputStream objectInputStream =
						new AnnotatedObjectInputStream(
							new ByteArrayInputStream(_data));

					_key = (Serializable)objectInputStream.readObject();
					_value = (Serializable)objectInputStream.readObject();
				}
				catch (ClassNotFoundException cnfe) {
					_log.error("Unable to deserialize object", cnfe);

					return null;
				}
			}

			return new Element(_key, _value);
		}

		@Override
		public void writeExternal(ObjectOutput objectOutput)
			throws IOException {

			ByteArrayOutputStream byteArrayOutputStream =
				new ByteArrayOutputStream();

			EhcacheObjectOutputStream ehcacheObjectOutputStream =
				new EhcacheObjectOutputStream(byteArrayOutputStream);

			ehcacheObjectOutputStream.writeObject(_key);
			ehcacheObjectOutputStream.writeObject(_value);

			ehcacheObjectOutputStream.close();

			objectOutput.writeObject(
				ehcacheObjectOutputStream.getDependentContextNames());

			byte[] serializedData = byteArrayOutputStream.toByteArray();

			objectOutput.writeInt(serializedData.length);
			objectOutput.write(serializedData, 0, serializedData.length);
		}

		private byte[] _data;
		private Set<String> _dependentContextNames;
		private Serializable _key;
		private Serializable _value;

	}

	private static class EhcacheHotDeployMessageListener
		extends HotDeployMessageListener {

		@Override
		protected void onDeploy(Message message) throws Exception {
			if (_unloadedEhcaches.isEmpty()) {
				return;
			}

			Set<Map.Entry<String, Set<EhcacheElement>>> entries =
				_unloadedEhcaches.entrySet();

			Iterator<Map.Entry<String, Set<EhcacheElement>>> itr =
				entries.iterator();

			while (itr.hasNext()) {
				Map.Entry<String, Set<EhcacheElement>> entry = itr.next();

				Ehcache ehcache = _cacheManager.getCache(entry.getKey());

				Set<EhcacheElement> ehcacheElements = entry.getValue();

				Iterator<EhcacheElement> elementItr =
					ehcacheElements.iterator();

				while (elementItr.hasNext()) {
					EhcacheElement ehcacheElement = elementItr.next();

					Element element = ehcacheElement.toElement();

					if (element != null) {
						ehcache.put(element, true);

						elementItr.remove();
					}
				}

				if (ehcacheElements.isEmpty()) {
					itr.remove();
				}
			}
		}

		private EhcacheHotDeployMessageListener(CacheManager cacheManager) {
			_cacheManager = cacheManager;
		}

		private final CacheManager _cacheManager;
	}

	private static class EhcacheObjectOutputStream extends ObjectOutputStream {

		public EhcacheObjectOutputStream(OutputStream outputStream)
			throws IOException {

			super(outputStream);
		}

		@Override
		protected void annotateClass(Class<?> clazz) throws IOException {
			ClassLoader classLoader = clazz.getClassLoader();

			String contextName = ClassLoaderPool.getContextName(classLoader);

			writeUTF(contextName);

			_dependentContextNames.add(contextName);
		}

		public Set<String> getDependentContextNames() {
			return _dependentContextNames;
		}

		private final Set<String> _dependentContextNames =
			new HashSet<String>();

	}

	private static class EhcacheStreamServerThread extends Thread {

		public EhcacheStreamServerThread(
			ServerSocket serverSocket, CacheManager cacheManager,
			List<String> cacheNames) {

			_serverSocket = serverSocket;
			_cacheManager = cacheManager;
			_cacheNames = cacheNames;

			setDaemon(true);
			setName(
				EhcacheStreamServerThread.class.getName() + " - " + cacheNames);
			setPriority(Thread.NORM_PRIORITY);
		}

		@Override
		public void run() {
			Socket socket = null;

			try {
				try {
					socket = _serverSocket.accept();
				}
				catch (SocketTimeoutException ste) {
					if (_log.isDebugEnabled()) {
						_log.debug(
							"Terminating the socket thread " + getName() +
								" that the client requested but never used");
					}

					return;
				}
				finally {
					_serverSocket.close();
				}

				socket.shutdownInput();

				ObjectOutputStream objectOutputStream = new ObjectOutputStream(
					socket.getOutputStream());

				for (String cacheName : _cacheNames) {
					Ehcache ehcache = _cacheManager.getCache(cacheName);

					if (ehcache == null) {
						EhcacheStreamBootstrapCacheLoader.setSkip();

						try {
							_cacheManager.addCache(cacheName);
						}
						finally {
							EhcacheStreamBootstrapCacheLoader.resetSkip();
						}

						continue;
					}

					objectOutputStream.writeObject(cacheName);

					List<Object> keys = ehcache.getKeys();

					for (Object key : keys) {
						if (!(key instanceof Serializable)) {
							if (_log.isWarnEnabled()) {
								_log.warn(
									"Key " + key + " is not serializable");
							}

							continue;
						}

						Element element = ehcache.get(key);

						if (element == null) {
							continue;
						}

						Object value = element.getObjectValue();

						if (!(value instanceof Serializable)) {
							if (_log.isWarnEnabled() && (value != null)) {
								_log.warn(
									"Value " + value + " is not serializable");
							}

							continue;
						}

						EhcacheElement ehcacheElement = new EhcacheElement(
							(Serializable)key, (Serializable)value);

						objectOutputStream.writeObject(ehcacheElement);
					}
				}

				objectOutputStream.writeObject(_COMMAND_SOCKET_CLOSE);

				objectOutputStream.close();
			}
			catch (Exception e) {
				throw new RuntimeException(e);
			}
			finally {
				if (socket != null) {
					try {
						socket.close();
					}
					catch (IOException ioe) {
						throw new RuntimeException(ioe);
					}
				}
			}
		}

		private CacheManager _cacheManager;
		private List<String> _cacheNames;
		private ServerSocket _serverSocket;

	}

	private static class SocketCacheServerSocketConfiguration
		implements ServerSocketConfigurator {

		@Override
		public void configure(ServerSocket serverSocket)
			throws SocketException {

			serverSocket.setSoTimeout(PropsValues.EHCACHE_SOCKET_SO_TIMEOUT);
		}

	}

}