/**
 * Copyright (c) 2000-2017 Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * pSubscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.verify;

import com.liferay.portal.kernel.dao.jdbc.DataAccess;
import com.liferay.portal.kernel.json.JSONArray;
import com.liferay.portal.kernel.json.JSONException;
import com.liferay.portal.kernel.json.JSONFactoryUtil;
import com.liferay.portal.kernel.json.JSONObject;
import com.liferay.portal.kernel.staging.Staging;
import com.liferay.portal.kernel.staging.StagingConstants;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.xml.Document;
import com.liferay.portal.kernel.xml.DocumentException;
import com.liferay.portal.kernel.xml.Element;
import com.liferay.portal.kernel.xml.SAXReaderUtil;
import com.liferay.portlet.dynamicdatamapping.util.DDMXMLUtil;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;

import java.util.Iterator;

/**
 * @author Joshua Gok
 */
public class VerifyPortalPreferences extends VerifyProcess {

	protected String convertStagingPreferencesToJSON(String preferences)
		throws Exception {

		Element newRootElement = SAXReaderUtil.createElement(
			"portlet-preferences");

		JSONArray stagingPreferencesJSONArray = _parsePortalPreferences(
			preferences, newRootElement);

		stagingPreferencesJSONArray = _filterStagingPreferencesJSONArray(
			stagingPreferencesJSONArray);

		Element stagingPreferencesXmlElement =
			_convertStagingPreferencesJSONArrayToXmlElement(
				stagingPreferencesJSONArray);

		if (stagingPreferencesXmlElement != null) {
			newRootElement.add(stagingPreferencesXmlElement);
		}

		Document newDocument = SAXReaderUtil.createDocument();

		newDocument.add(newRootElement);

		return DDMXMLUtil.formatXML(newDocument);
	}

	@Override
	protected void doVerify() throws Exception {
		updatePortalPreferences();
	}

	protected void updatePortalPreferences() throws Exception {
		Connection con = null;
		PreparedStatement ps = null;
		ResultSet rs = null;

		try {
			con = DataAccess.getUpgradeOptimizedConnection();

			ps = con.prepareStatement(
				"select portalPreferencesId, preferences from " +
					"PortalPreferences");

			rs = ps.executeQuery();

			while (rs.next()) {
				long portalPreferencesId = rs.getLong("portalPreferencesId");

				String preferences = rs.getString("preferences");

				if (preferences.contains(Staging.class.getName())) {
					updateUserStagingPreferences(
						portalPreferencesId, preferences);
				}
			}
		}
		finally {
			DataAccess.cleanUp(con, ps, rs);
		}
	}

	protected void updateUserStagingPreferences(
			long portalPreferencesId, String preferences)
		throws Exception {

		Connection con = null;
		PreparedStatement ps = null;

		try {
			con = DataAccess.getUpgradeOptimizedConnection();

			ps = con.prepareStatement(
				"update PortalPreferences set preferences = ? where " +
					"portalPreferencesId = ?");

			ps.setString(1, convertStagingPreferencesToJSON(preferences));
			ps.setLong(2, portalPreferencesId);
			ps.executeUpdate();
		}
		finally {
			DataAccess.cleanUp(con, ps);
		}
	}

	private Element _convertStagingPreferencesJSONArrayToXmlElement(
		JSONArray stagingPreferencesJSONArray) {

		Element preferenceElement = null;

		if (stagingPreferencesJSONArray.length() > 0) {
			preferenceElement = SAXReaderUtil.createElement("preference");

			Element nameElement = SAXReaderUtil.createElement("name");

			String stagingPreferencesName =
				Staging.class.getName() + StringPool.POUND +
					StagingConstants.STAGING_RECENT_LAYOUT_IDS_MAP;

			nameElement.setText(stagingPreferencesName);

			Element valueElement = SAXReaderUtil.createElement("value");

			valueElement.setText(stagingPreferencesJSONArray.toString());

			preferenceElement.add(nameElement);
			preferenceElement.add(valueElement);
		}

		return preferenceElement;
	}

	private JSONArray _filterStagingPreferencesJSONArray(
		JSONArray stagingPreferencesJSONArray) {

		JSONArray filteredStagingPreferencesJSONArray =
			JSONFactoryUtil.createJSONArray();

		for (int i = 0; i < stagingPreferencesJSONArray.length(); i++) {
			JSONObject jsonObject = stagingPreferencesJSONArray.getJSONObject(
				i);

			Iterator<String> iterator = jsonObject.keys();

			while (iterator.hasNext()) {
				if (!_isPreferenceStagingRecentLayoutIdsMap(iterator.next())) {
					filteredStagingPreferencesJSONArray.put(jsonObject);
				}
			}
		}

		return filteredStagingPreferencesJSONArray;
	}

	private boolean _isPreferenceStagingRecentLayoutIdsMap(String key) {
		return key.contains(StagingConstants.STAGING_RECENT_LAYOUT_IDS_MAP);
	}

	private JSONArray _parsePortalPreferences(
			String preferences, Element newRootElement)
		throws DocumentException, JSONException {

		JSONArray stagingPreferencesJSONArray =
			JSONFactoryUtil.createJSONArray();

		Document document = SAXReaderUtil.read(preferences);

		Element rootElement = document.getRootElement();

		Iterator<Element> iterator = rootElement.elementIterator();

		while (iterator.hasNext()) {
			Element preferenceElement = iterator.next();

			String preferenceName = preferenceElement.elementText("name");

			if (_isPreferenceStagingRecentLayoutIdsMap(preferenceName)) {
				String preferenceValue = preferenceElement.elementText("value");

				stagingPreferencesJSONArray = JSONFactoryUtil.createJSONArray(
					preferenceValue);
			}
			else if (!preferenceName.contains(Staging.class.getName())) {
				newRootElement.add(preferenceElement.createCopy());
			}
		}

		return stagingPreferencesJSONArray;
	}

}